/*
 * Copyright (C) 2003 Patrick Reynolds <reynolds@cs.duke.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 *      Free Software Foundation, Inc.
 *      59 Temple Place
 *      Suite 330
 *      Boston, MA 02111-1307
 *      USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "libppm.h"

#define PIX(x, y, c) img.data[3*((x) + (y)*w) + c]
#define RED 0
#define GREEN 1
#define BLUE 2

int get_rgb(char *param, short *r, short *g, short *b);
int char_to_dig(char c);

int main(int argc, char **argv) {
  short w, h, i, border;
  short adjust[3] = { 0, 0, 0 };
  struct ppm_image img;

  if (argc > 3 || argc < 2) {
    fprintf(stderr,
      "Usage:\n  %s bordersize [color] < input.ppm > output.ppm\n"
      "  color must be in RRGGBB (hex) format\n", argv[0]);
    exit(1);
  }

  if (argc == 3)
    if (get_rgb(argv[2], &adjust[0], &adjust[1], &adjust[2]) < 0) {
      fprintf(stderr, "%s: invalid color code: %s\n", argv[0], argv[2]);
      exit(1);
    }

  border = atoi(argv[1]);
  if (border <= 0) {
    fprintf(stderr, "%s: border too small\n", argv[0]);
    exit(1);
  }

  if (ppminit(&img) < 0) { fprintf(stderr, "ppminit failed\n"); return -1; }
  if (ppmread(stdin, &img) < 0) { fprintf(stderr, "ppmread failed\n"); return -1; }
  w = img.width;  h = img.height;
  if (border > w/2 || border > h/2) {
    fprintf(stderr, "%s: border too large\n", argv[0]);
    exit(1);
  }

  for (i=0; i<border; i++) {
    short j;

    /* horizontals */
    for (j=i; j<w-i; j++) {
      short k;
      for (k=RED; k<=BLUE; k++) {
        PIX(j, i, k) =
          adjust[k] + (PIX(j, i, k) - adjust[k]) * (i + 1) / (border + 1);
        PIX(j, h-i-1, k) =
          adjust[k] + (PIX(j, h-i-1, k) - adjust[k]) * (i + 1) / (border + 1);
      }
    }

    /* verticals */
    for (j=i+1; j<h-i-1; j++) {
      short k;
      for (k=RED; k<=BLUE; k++) {
        PIX(i, j, k) =
          adjust[k] + (PIX(i, j, k) - adjust[k]) * (i + 1) / (border + 1);
        PIX(w-i-1, j, k) =
          adjust[k] + (PIX(w-i-1, j, k) - adjust[k]) * (i + 1) / (border + 1);
      }
    }
  }

  if (ppmwrite(stdout, &img) < 0) { fprintf(stderr, "ppmwrite failed\n"); return -1; }
  ppmdestroy(&img);
  return 0;
}

int get_rgb(char *param, short *r, short *g, short *b) {
  short n[6];
  short i;

  if (strlen(param) != 6)
    return -1;
  for (i=0; i<6; i++)
    if ((n[i] = char_to_dig(param[i])) < 0)
      return -1;
  *r = n[0]*16 + n[1];
  *g = n[2]*16 + n[3];
  *b = n[4]*16 + n[5];
  return 0;
}

int char_to_dig(char c) {
  if (c >= '0' && c <= '9')
    return c - '0';
  else if (c >= 'a' && c <= 'f')
    return c - 'a' + 10;
  else if (c >= 'A' && c <= 'F')
    return c - 'A' + 10;
  else
    return -1;
}
