/*
 * Copyright (C) 2003 Patrick Reynolds <reynolds@cs.duke.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 *      Free Software Foundation, Inc.
 *      59 Temple Place
 *      Suite 330
 *      Boston, MA 02111-1307
 *      USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "libppm.h"

static void usage(const char *prog);

int main(int argc, char *argv[]) {
  float scale = 1.0;
  struct ppm_image img;
  unsigned char *p;
  int w, h, nw = 0, nh = 0;

  if (argc != 2) usage(argv[0]);

  if (ppminit(&img) < 0) { fprintf(stderr, "ppminit failed\n"); return -1; }
  if (ppmread(stdin, &img) < 0) { fprintf(stderr, "ppmread failed\n"); return -1; }
  w = img.width;  h = img.height;
	if (!strncasecmp(argv[1], "width=", 6)) {
		nw = atoi(argv[1]+6);
		nh = h * nw/w;
	} else if (!strncasecmp(argv[1], "height=", 7)) {
		nh = atoi(argv[1]+7);
		nw = w * nh/h;
	} else if (!strncasecmp(argv[1], "fit=", 4)) {
		p = strchr(argv[1]+4, 'x');
		*p = '\0';
		p++;
		nw = atoi(argv[1]+4);
		nh = atoi(p);
		if (nw*h > nh*w) /* height ratio is larger */
			nw = w * nh/h;
		else
			nh = h * nw/w;
	}
	else if ((p = strchr(argv[1], 'x')) != NULL) {
    fprintf(stderr, "Using WxH mode\n");
    *p = '\0';
    p++;
    nw = atoi(argv[1]);
    nh = atoi(p);
  } else {
    scale = atof(argv[1]);
		nw = w * scale;
		nh = h * scale;
  }
	if (nw < 1 || nw > 2500 || nh < 1 || nh > 2000) {
		fprintf(stderr, "%dx%d are not reasonable dimensions\n", nw, nh);
		exit(1); 
	}
  fprintf(stderr, "Scaling from %dx%d to %dx%d\n", w, h, nw, nh);
  if (ppmscale(&img, nw, nh) < 0) { fprintf(stderr, "ppmscale failed\n"); return -1; }
  if (ppmwrite(stdout, &img) < 0) { fprintf(stderr, "ppmwrite failed\n"); return -1; }
  ppmdestroy(&img);
  return 0;
}

static void usage(const char *prog) {
	fprintf(stderr, "Usage:\n");
	fprintf(stderr, "  %s scalefactor\n", prog);
	fprintf(stderr, "  %s WIDTHxHEIGHT\n", prog);
	fprintf(stderr, "  %s height=N\n", prog);
	fprintf(stderr, "  %s width=N\n", prog);
	fprintf(stderr, "  %s fit=WIDTHxHEIGHT\n", prog);
	exit(1);
}
